import inspect
import time

ACCEPTABLE_FUNCTION_LENGTH = 20

time_sleep = time.sleep


def scaling_sleep(secs):
    # Inspecting the stack to retrieve the source code of the function/module where this was called
    caller_source = inspect.getsourcelines(inspect.stack()[1][0])[0]

    superfluous_space = len([line for line in caller_source if len(line.strip()) == 0]) / len(caller_source)
    long_function_multiplier = max(len(caller_source), ACCEPTABLE_FUNCTION_LENGTH) / ACCEPTABLE_FUNCTION_LENGTH

    secs = secs * (1 + superfluous_space) * long_function_multiplier
    return time_sleep(secs)


time.sleep = scaling_sleep



import builtins
import itertools
import random
import distance

bool_like_names = [''.join(i) for i in itertools.chain(itertools.product("truefals", repeat=4),
                                                       itertools.product("truefals", repeat=5))]

threshold = 5
bool_truthiness = {}
for name in bool_like_names:
    distance_sum = distance.levenshtein(name, 'true') + distance.levenshtein(name, 'false')
    if distance_sum < threshold and name not in ['true', 'false']:
        bool_truthiness[name] = distance.levenshtein(name, 'false') / (threshold - 1)

for name in bool_truthiness:
    setattr(builtins, name.title(), random.random() < bool_truthiness[name])



import builtins

reversed_builtin = reversed


def rotate_the_board(caller_globals):
    """Attempts to write reversed variables to the globals state the caller passes"""
    for v in caller_globals.copy():
        try:
            if isinstance(caller_globals[v], str):
                reversed_value = ''.join(reversed_builtin(locals()[v]))
            else:
                var_class = caller_globals[v].__class__
                reversed_value = var_class(reversed_builtin(caller_globals[v]))
            setattr(builtins, ''.join(reversed_builtin(v)), reversed_value)
        except Exception:
            pass


class DidNotRotateBoardException(Exception):
    pass


# Replace reversed with a lambda that throws an exception using dark magic - turns out it causes a lot of
# trouble though.
# noinspection PyTypeChecker
# builtins.reversed = lambda obj: (_ for _ in ()).throw(DidNotRotateBoardException('reversed is cheating!'))



import builtins
import datetime
import time

from . import butterfinger_booleans, code_language_detection, looks_optimised_to_me, oxford_dictionary, esrever
from .esrever import rotate_the_board


def mischief_managed():
    """Undo most of what esoterrible changed in the session"""
    builtins.dict = oxford_dictionary.dict_builtin
    builtins.reversed = esrever.reversed_builtin
    time.sleep = looks_optimised_to_me.time_sleep
    datetime.datetime = code_language_detection.datetime_datetime
    for name in butterfinger_booleans.bool_truthiness:
        del name



import builtins
import random
import warnings
import PyDictionary

dict_builtin = dict


class OxfordDict(dict):

    def __init__(self, *args, **kwargs):

        self.dictionary = PyDictionary.PyDictionary()
        super().__init__(*args, **kwargs)

    def __getitem__(self, item):
        try:
            return super().__getitem__(item)
        except KeyError as ex:

            # stop overriding while using PyDictionary.meaning
            builtins.dict = dict_builtin

            with warnings.catch_warnings():
                warnings.simplefilter("ignore")

                meanings = self.dictionary.meaning(str(item))
                if meanings is None:
                    raise
                result = random.choice(list(meanings.values())[0])

            builtins.dict = OxfordDict
            return result


builtins.dict = OxfordDict



import datetime
import re
import sys
import os
import pytz
import pkg_resources


UK_WORDS = pkg_resources.resource_string('esoterrible.resources', 'uk_words.txt').decode("utf-8").split()
US_WORDS = pkg_resources.resource_string('esoterrible.resources', 'us_words.txt').decode("utf-8").split()

datetime_datetime = datetime.datetime


class InconsistentSpellingError(Exception):
    pass


class VeryBritishDatetime(datetime.datetime):
    london = pytz.timezone('Europe/London')

    def __new__(cls, *args, **kwargs):
        result = super().__new__(cls, *args, **kwargs)
        return cls.london.localize(result)


class VeryAmericanDatetime(datetime.datetime):
    # I'm not dedicated enough to fine-tune timezone based on regional dialects
    new_york = pytz.timezone('America/New_York')

    def __new__(cls, *args, **kwargs):
        result = super().__new__(cls, *args, **kwargs)
        return cls.new_york.localize(result)


def set_datetime_to_dialect_timezone(source_file_path):
    with open(source_file_path, 'r') as source_code_file:
        source_uk_words = []
        source_us_words = []
        source_code = source_code_file.read()

        # stripping punctuation from file
        source_code = re.sub(r'[^\w\s]', '', source_code)

        for word in source_code.split():
            if word in UK_WORDS:
                source_uk_words.append(word)
            if word in US_WORDS:
                source_us_words.append(word)

        if source_uk_words and source_us_words:

            if len(source_uk_words) > len(source_us_words):
                minority_of_words = source_uk_words
            else:
                minority_of_words = source_us_words

            raise InconsistentSpellingError(
                "American and British English? Make your mind up! You probably want to change the "
                f"spelling of these words: {', '.join(minority_of_words)}")
        elif source_uk_words:
            datetime.datetime = VeryBritishDatetime
        elif source_us_words:
            datetime.datetime = VeryAmericanDatetime
        else:
            print("Nothing very American or British about your code")


try:
    main_file_path = os.path.abspath(sys.modules['__main__'].__file__)
    set_datetime_to_dialect_timezone(main_file_path)
except AttributeError:
    print("Skipping code language detection as Python is running in interactive mode")
